#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import subprocess
import gobject
import pygtk
pygtk.require('2.0')
import gtk
import os
from os.path import join, getsize
import dbus
from dbus.mainloop.glib import DBusGMainLoop
#import gnomevfs 
import gio              #Gio sustituye a gnomevfs..
import gettext
import shutil
import filecmp
import re
import thread
import threading
import time


LOCALEDIR = "/usr/share/cga-usb-creator/"
PATH = "/usr/share/cga-usb-creator/"
gettext.bindtextdomain('cga-usb-creator', LOCALEDIR)
gettext.textdomain('cga-usb-creator')

import __builtin__
__builtin__._ = gettext.gettext 


def free_space(dev):
        try:
                stat = os.statvfs(dev)
        except:
                # This could get called in the event loop as we're shutting down, after
                # we've unmounted filesystems.
                return 0
        return stat.f_bsize * stat.f_bavail

def calcular_tamano(dir):
        try:
                total = 0
                for root, dirs, files in os.walk(dir):
                        relpath = root.replace(dir,'')
                        print relpath
                        if re.search("^\/\.", relpath) != None:
                                continue
                        tam = 0
                        for name in files:
                                if os.path.isfile(join(root, name)):
                                        tam = tam + getsize(join(root, name))
                        total = total + tam
                return total
        except OSError, msg:
                print msg
                return -1

def format_size(size):
        """Format a partition size."""
        # Taken from ubiquity's ubiquity/misc.py
        if size == "":
                size = 0
        if size < 1024:
                unit = 'B'
                factor = 1
        elif size < 1024 * 1024:
                unit = 'kB'
                factor = 1024
        elif size < 1024 * 1024 * 1024:
                unit = 'MB'
                factor = 1024 * 1024
        elif size < 1024 * 1024 * 1024 * 1024:
                unit = 'GB'
                factor = 1024 * 1024 * 1024
        else:
                unit = 'TB'
                factor = 1024 * 1024 * 1024 * 1024
        return '%.1f %s' % (float(size) / factor, unit)




class usbcreator_Backend:
        def __init__(self, frontend):
                self.frontend = frontend
                self.devices = {}
                DBusGMainLoop(set_as_default=True)
                self.bus = dbus.Bus(dbus.Bus.TYPE_SYSTEM)
                hal_obj = self.bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
                self.hal = dbus.Interface(hal_obj, 'org.freedesktop.Hal.Manager')

        def detectar_dispositivos(self):
                devices = self.hal.FindDeviceByCapability('storage')
                for device in devices:
                        print "\nNUEVO DISPOSITIVO"
                        dev_obj = self.bus.get_object('org.freedesktop.Hal', device)
                        d = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
                        if not self.es_disp_almacenamiento(d):
                                continue
                        print "ES VALIDO"
                        
                        # Comprueba si este dispositivo tiene un sistema de ficheros pero no tiene particiones
                        if d.GetProperty('block.is_volume'):
                                #if d.GetProperty('volume.fstype') == 'vfat':
                                self.incluir_dispositivo(d)
                        else: # En caso constrario, busca todas las particiones del dispositivo
                                children = self.hal.FindDeviceStringMatch('info.parent', device)
                                c = False
                                for child in children:
                                        dev_obj = self.bus.get_object('org.freedesktop.Hal', child)
                                        child = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
                                        if child.GetProperty('block.is_volume'):
                                                #if child.GetProperty('volume.fstype') == 'vfat':
                                                c = True
                                                self.incluir_dispositivo(child)

                                if not c and d.PropertyExists('storage.removable.media_size'):
                                        udi = d.GetProperty('info.udi')
                                        device = str(d.GetProperty('block.device'))
                                        self.devices[device] = {
                                        'label' : '',
                                        'fstype' : '',
                                        'uuid' : '',
                                        'mountpoint' : '',
                                        'udi' : udi,
                                        'free' : d.GetProperty('storage.removable.media_size'),
                                        'capacity' : d.GetProperty('storage.removable.media_size'),
                                        'device' : device
                                        }
                self.bus.add_signal_receiver(self.dispositivo_detectado, "DeviceAdded", "org.freedesktop.Hal.Manager", "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
                self.bus.add_signal_receiver(self.borrar_dispositivo, "DeviceRemoved", "org.freedesktop.Hal.Manager", "org.freedesktop.Hal", "/org/freedesktop/Hal/Manager")
        
        def incluir_dispositivo(self, dev):
                dev_obj = self.bus.get_object("org.freedesktop.Hal", dev.GetProperty('info.parent'))
                device = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
                d = device.GetProperty('block.device')
                
                if self.devices.has_key(d):
                        self.devices.pop(d)
                        self.frontend.device_removed(d, source=False)

                udi = dev.GetProperty('info.udi')
                mountpoint = str(dev.GetProperty('volume.mount_point'))
                device = str(dev.GetProperty('block.device'))
                self.devices[device] = {
                        #'parent' : str(dev.GetProperty('info.parent')).replace('_', ' '),
                        'label' : str(dev.GetProperty('volume.label')).replace(' ', '_'),
                        'fstype' : str(dev.GetProperty('volume.fstype')),
                        'uuid' : str(dev.GetProperty('volume.uuid')),
                        'mountpoint' : mountpoint,
                        'udi' : str(dev.GetProperty('info.udi')),
                        'free' : mountpoint and free_space(mountpoint) or 0,
                        'capacity' : dev.GetProperty('volume.size'),
                        'device' : device,
                        'logical' : str(d)
                }
                self.bus.add_signal_receiver(self.modificacion_estado, 'PropertyModified', 'org.freedesktop.Hal.Device', 'org.freedesktop.Hal', udi, path_keyword='udi')
                self.frontend.incluir_dispositivo(self.devices[device])

        def borrar_dispositivo(self, udi):
                d = None
                for device in self.devices.itervalues():
                        if device['udi'] == udi:
                                d = device['device']
                                break
                if d:
                        self.devices.pop(d)
                self.frontend.borrar_dispositivo(d)

        def dispositivo_detectado(self, udi):
                dev_obj = self.bus.get_object("org.freedesktop.Hal", udi)
                dev = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
                success = False
                if dev.PropertyExists('block.is_volume') and dev.GetProperty('block.is_volume'):
                        if (dev.PropertyExists('storage.bus') and dev.GetProperty('storage.bus') == 'usb') and dev.GetProperty('storage.removable'):
                                #if dev.GetProperty('volume.fstype') == 'vfat':
                                success = True
                        else:
                                p = dev.GetProperty('info.parent')
                                dev_obj = self.bus.get_object('org.freedesktop.Hal', p)
                                d = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
                                if d.PropertyExists('storage.bus'):
                                        #if dev.GetProperty('volume.fstype') == 'vfat':
                                        success = True
                        if success:
                                self.incluir_dispositivo(dev)
                                return
                
        def modificacion_estado(self, num_changes, change_list, udi):
                dev_obj = self.bus.get_object("org.freedesktop.Hal", udi)
                dev = dbus.Interface(dev_obj, "org.freedesktop.Hal.Device")
                mountpoint = str(dev.GetProperty('volume.mount_point'))
                device = str(dev.GetProperty('block.device'))
                self.devices[device] = {
                'label' : str(dev.GetProperty('volume.label')).replace(' ', '_'),
                'fstype' : str(dev.GetProperty('volume.fstype')),
                'uuid' : str(dev.GetProperty('volume.uuid')),
                'mountpoint' : mountpoint,
                'udi' : str(dev.GetProperty('info.udi')),
                'free' : mountpoint and free_space(mountpoint) or 0,
                'capacity' : dev.GetProperty('volume.size'),
                'device' : device,
                'logical' : str(dev.GetProperty('block.device'))
                }
                self.frontend.actualizar_listado(self.devices[device])

        def es_disp_almacenamiento(self, d):
                if d.GetProperty('storage.bus') not in ['usb', 'mmc']:
                        return False

                drive_types = ['disk',
                       'sd_mmc',
                       'smart_media',
                       'memory_stick',
                       'compact_flash']
                if d.GetProperty('storage.drive_type') in drive_types:
                        return True


class Percent(gobject.GObject, threading.Thread):

        __gsignals__ = {
                         "percent": ( gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE, (gobject.TYPE_STRING,))
                        }

        def __init__(self, stream):
                gobject.GObject.__init__(self)
                threading.Thread.__init__(self)
                self.stream = stream

        def emit(self, *args):
                gobject.idle_add(gobject.GObject.emit,self,*args)


        def run(self):
                while True:
                        self.line = self.stream.readline()
                        if len(self.line) == 0:
                                break
                        if len(self.line) <=4:
                                print self.line, 
                                self.emit("percent", self.line)
                                time.sleep(1)


class usbcreator_Frontend():

        def __init__(self):
                self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                self.window.set_position(gtk.WIN_POS_CENTER)
                self.window.set_border_width(10)
                self.window.set_size_request(500, 350)
                self.window.set_title(_('Creador de llavero USB Guadalinex Edu'))

                self.box = gtk.VBox(False, 10)
                self.window.add(self.box)

                self.window.connect("delete_event", self.delete_event)
                self.window.connect("destroy", self.destroy)

                # Lista de dispositivos
                # - Etiqueta de la lista
                self.et_lista = gtk.Label()
                self.et_lista.set_markup(_('<b>Dispositivos usb conectados</b>'))
                self.et_lista.set_use_markup(True)
                self.box.pack_start(self.et_lista, False, True, 0)
                # - Lista
                self.contenedor_lista = gtk.ScrolledWindow()
                self.contenedor_lista.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
                self.contenedor_lista.set_shadow_type(gtk.SHADOW_IN)
                self.listado = gtk.ListStore(str, str, str, str)
                self.vista_arbol_lista = gtk.TreeView()
                self.vista_arbol_lista.set_model(self.listado)
                self.contenido_columna_1 = gtk.CellRendererText()
                self.columna_1 = gtk.TreeViewColumn(_('Dispositivo'), self.contenido_columna_1, text=0)
                self.vista_arbol_lista.append_column(self.columna_1)
                self.contenido_columna_3 = gtk.CellRendererText()
                self.columna_3 = gtk.TreeViewColumn(_('Capacidad'), self.contenido_columna_3, text=2)
                self.vista_arbol_lista.append_column(self.columna_3)
                self.contenido_columna_4 = gtk.CellRendererText()
                self.columna_4 = gtk.TreeViewColumn(_('Etiqueta'), self.contenido_columna_4, text=3)
                self.vista_arbol_lista.append_column(self.columna_4)
                self.seleccion_vista = self.vista_arbol_lista.get_selection()
                self.seleccion_vista.set_mode(gtk.SELECTION_SINGLE)
                self.seleccion_vista.connect("changed", self.actualizar_info_dispositivo)
                self.contenedor_lista.add(self.vista_arbol_lista)
                self.contenedor_lista.show_all()
                self.box.pack_start(self.contenedor_lista, True, True, 0)
                # Fin lista de dispositivos
        
                # Cuadro informativo
                # -Cabecera
                self.hbox_info_cab = gtk.HBox(False, 5)
                self.et_cab_info = gtk.Label(_('Dispositivo seleccionado: '))
                self.et_cab_info_dev = gtk.Label()
                self.et_cab_info_dev.set_markup(_('<i>dispositivo no seleccionado</i>'))
                self.et_cab_info_dev.set_use_markup(True)
                self.hbox_info_cab.pack_start(self.et_cab_info, False, True, 0)
                self.hbox_info_cab.pack_start(self.et_cab_info_dev, False, True, 0)
                self.et_cab_info.show()
                self.et_cab_info_dev.show()
                self.hbox_info_cab.show()
                self.box.pack_start(self.hbox_info_cab, False, True, 0)
                                
                self.hbox_info_pm = gtk.HBox(False, 5)
                self.filechooserdialog = gtk.FileChooserButton("Seleccionar archivo", None)
                self.filechooserdialog.set_current_folder(os.path.expanduser("~"))
                self.filechooserdialog.connect("file-set", self.file_selected)
                self.filechooserdialog.set_sensitive(False)
                self.filtro = gtk.FileFilter()
                self.filtro.set_name(".iso")
                self.filtro.add_pattern("*.iso")
                self.filechooserdialog.add_filter(self.filtro)
                self.hbox_info_pm.pack_start(self.filechooserdialog, True, True, 0)
                self.filechooserdialog.show()
                self.hbox_info_pm.show()
                self.box.pack_start(self.hbox_info_pm, False, True, 0)
                # Fin de cuadro informativo
       
                # -Boton de crear
                self.hbox_boton_cop = gtk.HBox(False,5)
                self.b_iniciar_cop = gtk.Button("Crear")
                self.b_iniciar_cop.set_tooltip_text(_('Crear'))
                self.b_iniciar_cop.set_size_request(90, 30)
                self.b_iniciar_cop.connect("clicked", self.create_aviso)
                self.b_iniciar_cop.set_sensitive(False)
                self.hbox_boton_cop.pack_start(self.b_iniciar_cop, True, False, 0)
                self.b_iniciar_cop.show()
                self.hbox_boton_cop.show()
                self.box.pack_start(self.hbox_boton_cop, False, False, 0)
                

                self.vbox_spinner = gtk.VBox(False, 5) 
                self.spinner = gtk.Spinner()
                self.spinner.set_size_request(50, 50)
                self.label_spinner = gtk.Label("Operaciones")
                self.vbox_spinner.pack_start(self.spinner, True, False, 0)
                self.vbox_spinner.pack_start(self.label_spinner, True, False, 0)
                self.box.pack_start(self.vbox_spinner, False, False, 0)
                self.vbox_spinner.show()

                self.backend = usbcreator_Backend(self)
                self.backend.detectar_dispositivos()

                self.et_lista.show()
                self.box.show() 
                self.window.show()


        def create_aviso(self, widget, data=None):
                self.mensaje_aviso("Se dispone a borrar su dispositivo.\n¿Desea continuar?", "warning") 


        def file_selected(self, widget):
                #Se múitiplica por 1.05 para no tener problemas con usb de 2 gb
                if self.size_bytes > os.path.getsize(self.filechooserdialog.get_filename())*1.05:
                        self.b_iniciar_cop.set_sensitive(True)
                else:
                        self.mensaje_aviso("Es necesario un dispositivo de mayor tamaño.", "error")


        def actualizar_listado(self, disp):
                texto = None
                if disp['udi']:
                        drive = self.get_gnome_drive(disp['udi'])
                        if drive:
                                texto = drive.get_name()
                        else:
                                texto = disp['mountpoint']
                else:
                        texto = disp['filename']
                aux = self.vista_arbol_lista.get_model()
                iter = aux.get_iter_first()
                while iter is not None:
                        if aux.get_value(iter, 1) == disp['device']:
                                aux.set_value(iter, 0, texto)
                                aux.set_value(iter, 1, disp['device'])
                                aux.set_value(iter, 2, format_size(disp['capacity']))
                                aux.set_value(iter, 3, disp['label'])
                        iter = aux.iter_next(iter)


        def incluir_dispositivo(self, disp):
                texto = None
                if disp['udi']:
                        drive = self.get_gnome_drive(disp['udi'])
                        if drive:
                                texto = drive.get_name()
                        else:   
                                texto = disp['mountpoint']
                else:
                        texto = disp['filename']
                self.listado.append([texto, disp['device'], format_size(disp['capacity']), disp['label']])


        def borrar_dispositivo(self, disp):
                borrar = None
                aux = self.vista_arbol_lista.get_model()
                iter = aux.get_iter_first()
                while iter is not None:
                        if aux.get_value(iter, 1) == disp:
                                borrar = iter
                        iter = aux.iter_next(iter)
                if borrar  is not None:
                        aux.remove(borrar)


        def actualizar_info_dispositivo(self, widget, data = None):
                device = None
                (aux, iter) = self.seleccion_vista.get_selected()
                if iter is not None:
                        device = aux.get_value(iter, 1)
                else:
                        self.et_cab_info_dev.set_markup(_('<i>dispositivo no seleccionado</i>'))
                        self.filechooserdialog.set_sensitive(False)
                        self.b_iniciar_cop.set_sensitive(False)
                        
                if device is not None:
                        disp = self.backend.devices[device]
                        self.et_cab_info_dev.set_markup(aux.get_value(iter, 0))
                        self.filechooserdialog.set_sensitive(True)
                        self.dispositivo_seleccionado = aux.get_value(iter, 1)
                        self.size = aux.get_value(iter, 2)
                        self.device = disp['logical']
                        self.size_bytes = disp['capacity']
                        self.partition = disp['device']
 
        def get_gnome_drive(self, udi):
                monitor = gio.VolumeMonitor()
                for drive in monitor.get_connected_drives():
                        if drive.get_identifier(gio.FILE_ATTRIBUTE_MOUNTABLE_HAL_UDI) == udi:
                                return drive

        
        def delete_event(self, widget, data = None):
                gtk.main_quit()

        def destroy(self, widget, data = None):
                gtk.main_quit()


        
        def mensaje_aviso(self, msg, icono): 

                self.cadena = msg
 
                # Ventana del mensaje y contenedor
                self.warning_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
                self.warning_window.set_position(gtk.WIN_POS_CENTER)
                self.warning_window.set_title(_('Creador de llavero USB Guadalinex Edu'))
                self.warning_window.set_border_width(10)
                self.warning_window.set_deletable(False)
                self.warning_window.set_modal(True)
                self.window.set_sensitive(False)
                self.warning_window.set_transient_for(self.window)
                self.warning_window.set_resizable(False)
                self.warning_window.connect("delete_event", self.delete_event_warning)
                self.warning_window.connect("destroy", self.destroy_warning)
                self.vbox = gtk.VBox(False, 30)
                self.panel = gtk.Fixed()
                self.vbox.pack_start(self.panel, True, True, 0)
                self.warning_window.add(self.vbox)

                # Mensaje adaptado
                self.mensaje = gtk.Label()
                if msg != None:
                        self.mensaje.set_text("%s" % self.cadena)
                self.mensaje.set_justify(gtk.JUSTIFY_CENTER)

                # Cabecera de la ventana de aviso
                self.imagen_aviso = gtk.Image()
                self.et_aviso = gtk.Label()

                if icono == "warning":
                        self.boton_cancelar = gtk.Button(_('Cancelar'))
                        self.boton_cancelar.set_size_request(90,30)
                        self.boton_cancelar.connect("clicked", self.delete_event_warning, None)
                        self.boton_aceptar = gtk.Button(_('Aceptar'))
                        self.boton_aceptar.connect("clicked", self.run_thread_create_usb, None)
                        self.boton_aceptar.set_size_request(90,30)
                        self.imagen_aviso.set_from_stock(gtk.STOCK_DIALOG_WARNING, gtk.ICON_SIZE_DIALOG)
                        self.panel.put(self.imagen_aviso, 170, 0)
                        self.panel.put(self.mensaje, 70, 60)
                        self.panel.put(self.boton_cancelar, 190, 105)
                        self.panel.put(self.boton_aceptar, 290, 105)
                        self.boton_cancelar.show()
                        self.boton_aceptar.show()

                elif icono == "error":
                        self.imagen_aviso.set_from_stock(gtk.STOCK_DIALOG_ERROR, gtk.ICON_SIZE_DIALOG)
                        self.boton_cancelar = gtk.Button(_('Cancelar'))
                        self.boton_cancelar.set_size_request(90,30)
                        self.boton_cancelar.connect("clicked", self.restart, None)
                        self.panel.put(self.imagen_aviso, 170, 0)
                        self.panel.put(self.mensaje, 45, 60)
                        self.panel.put(self.boton_cancelar, 290, 105)
                        self.boton_cancelar.show()

                elif icono == "success":
                        self.boton_no = gtk.Button(_('No'))
                        self.boton_no.connect("clicked", gtk.main_quit)
                        self.boton_yes = gtk.Button(_('Sí'))
                        self.boton_yes.connect("clicked", self.restart, None)     
                        self.boton_no.set_size_request(90,30)
                        self.boton_yes.set_size_request(90,30)

                        self.imagen_aviso.set_from_stock(gtk.STOCK_APPLY, gtk.ICON_SIZE_DIALOG)
                        self.panel.put(self.imagen_aviso, 170, 0)
                        self.panel.put(self.mensaje, 70, 60)
                        self.panel.put(self.boton_no, 190, 105)
                        self.panel.put(self.boton_yes, 290, 105)
                        self.boton_no.show()
                        self.boton_yes.show()                        

                # Dibujar la ventana de aviso
                self.warning_window.set_size_request(400, 150)
                self.imagen_aviso.show()
                self.mensaje.show()
                self.panel.show()
                self.vbox.show()
                self.warning_window.show()


        def restart(self, widget, data=None):
                self.warning_window.hide()
                self.label_spinner.hide()
                self.spinner.hide()
                self.window.set_sensitive(True)
                self.window.set_deletable(True)
                self.vista_arbol_lista.set_sensitive(True)
                self.filechooserdialog.set_current_folder(os.path.expanduser("~"))
                self.filechooserdialog.set_sensitive(True)
                self.et_cab_info_dev.set_sensitive(True)

                
        def delete_event_warning(self, widget, data=None):
                self.warning_window.hide()
                self.window.set_sensitive(True)
                self.b_iniciar_cop.set_sensitive(False)  


        def destroy_warning(self, widget, data=None):
                self.warning_window.destroy()


        def run_thread_create_usb(self, widget, data=None):
                self.window.set_deletable(False)
                thread = threading.Thread(target=self.create_usb)
                thread.start()
 

        def create_usb(self):
                self.window.set_sensitive(True)
                self.warning_window.hide()
                pincho = self.device
                partition = self.partition
                imagen = self.filechooserdialog.get_filename()
                self.et_cab_info_dev.set_sensitive(False)
                self.vista_arbol_lista.set_sensitive(False)
                self.filechooserdialog.set_sensitive(False)
                self.b_iniciar_cop.set_sensitive(False)
                self.spinner.start()
                self.spinner.show()
                self.label_spinner.set_text("Desmontando dispositivo")
                self.label_spinner.show()
                return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/desmontar_usb.sh", pincho])
                if return_code != 0:
                        self.mensaje_aviso("Error al desmontar el dispositivo.", "error")
                        return 1
                self.label_spinner.set_text("Montando imagen .iso")
                return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/montar_imagen.sh", imagen])
                if return_code != 0:
                        self.mensaje_aviso("Error al montar la imagen.", "error")
                        return 1
                self.label_spinner.set_text("Borrando particiones")
                return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/delete_partition.sh", pincho, partition])
                if return_code != 0:
                        self.mensaje_aviso("Error al borrar particiones.", "error")
                        return 1
                self.label_spinner.set_text("Sincronizando")
                return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/sync.sh"])
                if return_code != 0:
                        self.mensaje_aviso("Error al sincronizar.", "error")
                        return 1
                return_code = subprocess.Popen(["sudo", "/usr/share/cga-usb-creator/pv.sh"], stdout=subprocess.PIPE)
                reader = Percent(return_code.stdout)
                reader.connect("percent", self.update_percent)
                reader.start()
                return_code.wait()
                reader.join()
                if return_code.returncode != 0:
                        self.mensaje_aviso("Error al crear la imagen.", "error")
                        return 1
                self.label_spinner.set_text("Finalizando proceso")
                return_code = subprocess.call(["sudo", "/usr/share/cga-usb-creator/finished_imagen.sh", pincho])
                if return_code != 0:
                        self.mensaje_aviso("Error al finalizar el proceso.", "error")
                        return 1
                self.spinner.stop()
                self.spinner.hide()
                self.mensaje_aviso("El proceso ha terminado correctamente.\n¿Desea crear otro?", "success")


        def update_percent(self, sender, percent):
                self.label_spinner.set_text("Creando imagen %s%%" %percent.strip())


def main():
        gtk.gdk.threads_init()
        front = usbcreator_Frontend()

if __name__ =="__main__":
        main()
        gtk.main()  
